import argparse
import re
import shlex
from invoke import UnexpectedExit
from termcolor import colored

from .config import load_config
from .handlers import action_custom_server_command, action_custom_local_command
from .server_session import enter_live_server
from .colored_print import colored_print
from .genete_example import generate_local_config


def replace_placeholders(command, args):
    for i, arg in enumerate(args, start=1):
        command = re.sub(rf"\${i}\b", f'"{arg}"', command)
    return command


def main():
    parser = argparse.ArgumentParser(description="Project Commands")
    settings = load_config()

    parser.add_argument("command", nargs='+', help="Command to execute")
    args = parser.parse_args()

    full_command = " ".join(args.command)
    parsed_command = shlex.split(full_command)

    cmd_key = parsed_command[0]
    cmd_args = parsed_command[1:]

    if cmd_key == 'sv':
        enter_live_server(settings)
        return
    if cmd_key == 'ex':
        generate_local_config()
        return

    server_handler = settings.server_commands.get(cmd_key)
    local_handler = settings.local_commands.get(cmd_key)

    if server_handler:
        server_handler = replace_placeholders(server_handler, cmd_args)
        if '$' in server_handler:
            colored_print(f"Invalid command arguments: {cmd_key}", 'red')
            return
        try:
            action_custom_server_command(f"cd {settings.project.path} && {server_handler}", settings=settings)
        except UnexpectedExit:
            pass

    elif local_handler:
        local_handler = replace_placeholders(local_handler, cmd_args)
        if '$' in local_handler:
            colored_print(f"Invalid command arguments: {cmd_key}", 'red')
            return
        try:
            action_custom_local_command(local_handler, settings=settings)
        except Exception as e:
            colored_print(str(e), color='red')

    else:
        message = f'Command "{cmd_key}" not found'
        colored_message = colored(message, 'red', attrs=['bold'])
        print(colored_message)


if __name__ == '__main__':
    main()



import os, subprocess

from fabric import Connection

from .config import load_config
from .colored_print import colored_print


def action_custom_server_command(terminal_command: str, settings=None, server=None):
    """
    Connects to the server, and do given commands
    """
    settings = settings if settings else load_config()
    server_host = settings.server.host
    server_user = settings.server.user
    server_password = settings.server.password
    conn = server if server else Connection(
        host=server_host,
        user=server_user,
        connect_kwargs={"password": server_password}
    )
    with conn:
        result = conn.run(terminal_command)
        colored_print(result.stdout, 'green')


def action_custom_local_command(terminal_command: str, settings=None, server=None):
    """
    Executes a local shell command.
    """
    result = subprocess.run(terminal_command, shell=True, capture_output=True, text=True)
    if result.stdout:
        colored_print(result.stdout.strip(), 'green')
    if result.stderr:
        colored_print(result.stderr.strip(), 'red')



from dataclasses import dataclass
from typing import List, Dict, Optional, Callable


@dataclass
class Server:
    host: Optional[str]
    port: Optional[int]
    user: Optional[str]
    password: Optional[str]


@dataclass
class Project:
    path: Optional[str]



@dataclass
class Tez:
    server: Server
    project: Project
    server_commands: Dict[str, str]
    local_commands: Dict[str, 10]



import os, pyfiglet

from fabric import Connection
from invoke import UnexpectedExit
from termcolor import colored


from .handlers import action_custom_server_command
from .colored_print import colored_print


def enter_live_server(settings):
    """
    Connects to the server, provides an interactive shell experience, and allows directory navigation,
    command execution, and other shell commands like clear and exit.
    """
    server_host = settings.server.host
    server_user = settings.server.user
    server_password = settings.server.password
    try:
        # Establish connection
        with Connection(
                host=server_host,
                user=server_user,
                connect_kwargs={"password": server_password}
        ) as conn:
            colored_print(f'Connecting to {server_host} as {server_user}...', 'cyan')

            # Get initial directory
            current_path = conn.run("pwd", hide=True).stdout.strip()
            os.system('clear')
            r = pyfiglet.figlet_format(f'Success')
            colored_print(r, 'green')

            while True:
                try:
                    command_input = input(colored(f"{server_user}@{server_host}:{current_path}> ", 'cyan'))
                    if command_input.lower() in ("q", "exit"):
                        colored_print('Exited Tez server', 'red')
                        break
                    elif command_input.lower() == "clear":
                        os.system('clear')
                        continue
                    elif command_input.startswith("cd "):
                        new_dir = command_input.split(' ', 1)[1]
                        try:
                            test_dir = conn.run(f"cd {os.path.join(current_path, new_dir)} && pwd",
                                                hide=True).stdout.strip()
                            current_path = test_dir
                        except UnexpectedExit:
                            colored_print(f"Directory not found: {new_dir}", "red")
                        continue
                    terminal_command = settings.server_commands.get(command_input)
                    if terminal_command:
                        action_custom_server_command(f"cd {settings.project.path} && {terminal_command}", settings=settings,
                                              server=conn)
                    else:
                        try:
                            result = conn.run(f"cd {current_path} && {command_input}", hide=True)
                            print(result.stdout.strip())
                        except Exception as e:
                            colored_print(f"{e}", 'red')

                except KeyboardInterrupt:
                    colored_print("\nCommand canceled (Ctrl+C)", "yellow")
                except EOFError:
                    colored_print("\nExited Tez server (Ctrl+D)", "red")
                    continue

    except Exception as e:
        colored_print(f"Error connecting to the server: {e}", 'red')



import os
from pathlib import Path
from .colored_print import colored_print
from .config import load_config


def generate_local_config():
    """
    Generates the .tez configuration file in the home directory
    and writes the provided configuration for the server and project.
    """

    # Path to the home directory
    home_dir = Path(os.getcwd())
    tez_file_path = home_dir / ".tez"

    # Check if the home directory is correct
    print(f"Home directory: {home_dir}")

    # Configuration content
    tez_content = f"""
    [server]
SERVER_HOST=
SERVER_USER=
SERVER_PASSWORD=
SERVER_PORT=

[project]
PROJECT_PATH=

[local-commands]
view=ls

[server-commands]
s=ls
    """

    # Write to .tez file
    try:
        with open(tez_file_path, "w") as tez_file:
            tez_file.write(tez_content.strip())
        colored_print(f"Generated example .tez configuration file at: {tez_file_path}", "green")
    except Exception as e:
        print(f"Failed to write .tez file: {e}")

    # Rest of the generate logic (e.g., service and nginx config generation)
    # Continue with existing logic...



import configparser
import os

from .schemas import Tez, Server, Project


def load_config() -> Tez:
    try:
        config_file_name = ".tez"
        location = os.path.join(os.getcwd(), config_file_name)

        if not os.path.exists(location):
            raise FileNotFoundError(f"Configuration file '{config_file_name}' not found.")

        config = configparser.ConfigParser()
        config.read(location)

        server_config = {
            "host": config.get("server", "SERVER_HOST", fallback=None),
            "user": config.get("server", "SERVER_USER", fallback=None),
            "password": config.get("server", "SERVER_PASSWORD", fallback=None),
            "port": config.getint("server", "SERVER_PORT", fallback=None),
        }

        server_commands = {}
        if config.has_section("server-commands"):
            server_commands = {command: terminal_command for command, terminal_command in
                               config.items("server-commands")}

        local_commands = {}
        if config.has_section("local-commands"):
            local_commands = {command: terminal_command for command, terminal_command in config.items("local-commands")}
        project_config = {
            "path": config.get("project", "PROJECT_PATH", fallback=None),
        }

        return Tez(
            server=Server(**server_config),
            server_commands=server_commands,
            local_commands=local_commands,
            project=Project(**project_config),
        )
    except Exception as e:
        print(f"Error loading config: {e}")
        return Tez(
            server=Server(host=None, port=None, user=None, password=None),
            project=Project(path=None),
            server_commands={},
            local_commands={}
        )
